@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/pathnames.h,v 3.17 1998/09/13 13:51:03 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/pathnames.h,v 3.19 2010/05/07 16:19:04 christos Exp $ */
 /*
  * pathnames.h: Location of things to find
  */
@@ -14,11 +14,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -96,6 +92,10 @@
 # define _PATH_TCSHELL		"/local/bin/tcsh"	/* use ram disk */
 #endif /* _MINIX && !_PATH_TCSHELL */
 
+#if defined(__linux__) && !defined(_PATH_TCSHELL)
+# define _PATH_TCSHELL		"/bin/tcsh"
+#endif /* __linux__ && !_PATH_TCSHELL */
+
 #if defined(__EMX__) && !defined(_PATH_DEVNULL)
 # define _PATH_DEVNULL		"nul"
 #endif /* __EMX__ && !_PATH_DEVNULL */
@@ -145,7 +145,7 @@
 #endif /* !_PATH_CSHELL */
 
 #ifndef _PATH_TCSHELL
-#  define _PATH_TCSHELL		"/usr/local/bin/tcsh"
+# define _PATH_TCSHELL		"/usr/local/bin/tcsh"
 #endif /* !_PATH_TCSHELL */
 
 #ifndef _PATH_BIN_LOGIN
