@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/sh.lex.c,v 3.84 2010/05/12 15:10:48 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.lex.c,v 3.86 2010/05/17 19:36:45 christos Exp $ */
 /*
  * sh.lex.c: Lexical analysis into tokens
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$tcsh: sh.lex.c,v 3.84 2010/05/12 15:10:48 christos Exp $")
+RCSID("$tcsh: sh.lex.c,v 3.86 2010/05/17 19:36:45 christos Exp $")
 
 #include "ed.h"
 
@@ -460,7 +460,13 @@ getC1(int flag)
 		setexclp(exclnxt->word);
 	    continue;
 	}
-	c = readc(0);
+	c = readc(1);
+
+	/* Catch EOF in the middle of a line.  (An EOF at the beginning of
+	 * a line would have been processed by the readc(0) in lex().) */
+	if (c == CHAR_ERR)
+	    stderror(ERR_EOF);
+
 	if (c == '$' && (flag & DODOL)) {
 	    getdol();
 	    continue;
@@ -672,11 +678,11 @@ getexcl(Char sc)
     eChar c;
 
     if (sc == 0) {
-	sc = getC(0);
-	if (sc != '{') {
-	    ungetC(sc);
-	    sc = 0;
-	}
+	c = getC(0);
+	if (c == '{')
+	    sc = (Char) c;
+	else
+	    ungetC(c);
     }
     quesarg = -1;
 
@@ -693,7 +699,7 @@ getexcl(Char sc)
 	for (ip = hp->next->next; ip != hp->prev; ip = ip->next)
 	    dol++;
     left = 0, right = dol;
-    if (sc == HISTSUB) {
+    if (sc == HISTSUB && HISTSUB != '\0') {
 	ungetC('s'), unreadc(HISTSUB), c = ':';
 	goto subst;
     }
@@ -726,7 +732,7 @@ subst:
     exclc = right - left + 1;
     while (--left >= 0)
 	hp = hp->next;
-    if (sc == HISTSUB || c == ':') {
+    if ((sc == HISTSUB && HISTSUB != '\0') || c == ':') {
 	do {
 	    hp = getsub(hp);
 	    c = getC(0);
@@ -1147,7 +1153,7 @@ gethent(Char sc)
     int     event;
     int    back = 0;
 
-    c = sc == HISTSUB ? (eChar)HIST : getC(0);
+    c = (sc == HISTSUB && HISTSUB != '\0') ? (eChar)HIST : getC(0);
     if (c == (eChar)HIST) {
 	if (alhistp)
 	    return (alhistp);
